@@ -1,12 +1,12 @@
 use std::{
-    ffi::{CStr, OsStr, OsString},
+    ffi::{c_char, c_int, c_long, CStr, OsStr, OsString},
     os::{
         fd::{AsRawFd, BorrowedFd},
         unix::prelude::OsStrExt,
     },
 };
 
-pub fn cerr<Int: Copy + TryInto<libc::c_long>>(res: Int) -> std::io::Result<Int> {
+pub fn cerr<Int: Copy + TryInto<c_long>>(res: Int) -> std::io::Result<Int> {
     match res.try_into() {
         Ok(-1) => Err(std::io::Error::last_os_error()),
         _ => Ok(res),
@@ -23,15 +23,15 @@ extern "C" {
         link_name = "__errno"
     )]
     #[cfg_attr(target_os = "linux", link_name = "__errno_location")]
-    fn errno_location() -> *mut libc::c_int;
+    fn errno_location() -> *mut c_int;
 }
 
-pub fn set_errno(no: libc::c_int) {
+pub fn set_errno(no: c_int) {
     // SAFETY: errno_location is a thread-local pointer to an integer, so we are the only writers
     unsafe { *errno_location() = no };
 }
 
-pub fn sysconf(name: libc::c_int) -> Option<libc::c_long> {
+pub fn sysconf(name: c_int) -> Option<c_long> {
     set_errno(0);
     // SAFETY: sysconf will always respond with 0 or -1 for every input
     cerr(unsafe { libc::sysconf(name) }).ok()
@@ -44,7 +44,7 @@ pub fn sysconf(name: libc::c_int) -> Option<libc::c_long> {
 /// # Safety
 /// This function assumes that the pointer is either a null pointer or that
 /// it points to a valid NUL-terminated C string.
-pub unsafe fn string_from_ptr(ptr: *const libc::c_char) -> String {
+pub unsafe fn string_from_ptr(ptr: *const c_char) -> String {
     if ptr.is_null() {
         String::new()
     } else {
@@ -59,7 +59,7 @@ pub unsafe fn string_from_ptr(ptr: *const libc::c_char) -> String {
 /// # Safety
 /// This function assumes that the pointer is either a null pointer or that
 /// it points to a valid NUL-terminated C string.
-pub unsafe fn os_string_from_ptr(ptr: *const libc::c_char) -> OsString {
+pub unsafe fn os_string_from_ptr(ptr: *const c_char) -> OsString {
     if ptr.is_null() {
         OsString::new()
     } else {
@@ -69,7 +69,10 @@ pub unsafe fn os_string_from_ptr(ptr: *const libc::c_char) -> OsString {
     }
 }
 
-fn fstat_mode_set(fildes: &BorrowedFd, mask: libc::mode_t) -> bool {
+fn fstat_mode_any<const MASK: libc::mode_t>(fildes: &BorrowedFd) -> bool {
+    // FIXME put this in a const block once the MSRV has been bumped enough
+    assert!(MASK & libc::S_IFMT == MASK);
+
     // The Rust standard library doesn't have FileTypeExt on Std{in,out,err}, so we
     // can't just use FileTypeExt::is_char_device and have to resort to libc::fstat.
     let mut maybe_stat = std::mem::MaybeUninit::<libc::stat>::uninit();
@@ -80,16 +83,17 @@ fn fstat_mode_set(fildes: &BorrowedFd, mask: libc::mode_t) -> bool {
         let mode = unsafe { maybe_stat.assume_init() }.st_mode;
 
         // To complicate matters further, the S_ISCHR macro isn't in libc as well.
-        (mode & libc::S_IFMT) == mask
+        (mode & MASK) != 0
     } else {
         false
     }
 }
+
 /// Rust's standard library IsTerminal just directly calls isatty, which
 /// we don't want since this performs IOCTL calls on them and file descriptors are under
 /// the control of the user; so this checks if they are a character device first.
 pub fn safe_isatty(fildes: BorrowedFd) -> bool {
-    let is_char_device = fstat_mode_set(&fildes, libc::S_IFCHR);
+    let is_char_device = fstat_mode_any::<{ libc::S_IFCHR }>(&fildes);
 
     if is_char_device {
         // SAFETY: isatty will return 0 or 1
@@ -99,30 +103,32 @@ pub fn safe_isatty(fildes: BorrowedFd) -> bool {
     }
 }
 
-/// Check whether the file descriptor is a pipe
-pub fn is_fifo(fildes: BorrowedFd) -> bool {
-    fstat_mode_set(&fildes, libc::S_IFIFO)
+/// Check whether the file descriptor is a pipe or socket
+pub fn is_fifo_or_sock(fildes: BorrowedFd) -> bool {
+    fstat_mode_any::<{ libc::S_IFIFO | libc::S_IFSOCK }>(&fildes)
 }
 
 #[allow(clippy::undocumented_unsafe_blocks)]
 #[cfg(test)]
 mod test {
+    use std::ffi::c_char;
+
     use super::{os_string_from_ptr, string_from_ptr};
 
     #[test]
     fn miri_test_str_to_ptr() {
         let strp = |ptr| unsafe { string_from_ptr(ptr) };
         assert_eq!(strp(std::ptr::null()), "");
-        assert_eq!(strp("\0".as_ptr() as *const libc::c_char), "");
-        assert_eq!(strp("hello\0".as_ptr() as *const libc::c_char), "hello");
+        assert_eq!(strp("\0".as_ptr() as *const c_char), "");
+        assert_eq!(strp("hello\0".as_ptr() as *const c_char), "hello");
     }
 
     #[test]
     fn miri_test_os_str_to_ptr() {
         let strp = |ptr| unsafe { os_string_from_ptr(ptr) };
         assert_eq!(strp(std::ptr::null()), "");
-        assert_eq!(strp("\0".as_ptr() as *const libc::c_char), "");
-        assert_eq!(strp("hello\0".as_ptr() as *const libc::c_char), "hello");
+        assert_eq!(strp("\0".as_ptr() as *const c_char), "");
+        assert_eq!(strp("hello\0".as_ptr() as *const c_char), "hello");
     }
 
     #[test]
