@@ -12,8 +12,8 @@ use std::{
     io,
     os::unix::ffi::OsStrExt,
     os::unix::process::CommandExt,
-    path::Path,
-    process::Command,
+    path::{Path, PathBuf},
+    process::{self, Command},
     time::Duration,
 };
 
@@ -24,13 +24,12 @@ use crate::{
     exec::no_pty::exec_no_pty,
     log::{dev_info, dev_warn, user_error},
     system::{
-        _exit,
+        ForkResult, Group, User, _exit, fork,
         interface::ProcessId,
-        kill, killpg, mark_fds_as_cloexec, set_target_user,
-        signal::{consts::*, signal_name, SignalNumber, SignalSet},
+        kill, killpg, mark_fds_as_cloexec, set_target_user, setpgid,
+        signal::{consts::*, signal_name, SignalNumber, SignalSet, SignalsState},
         term::UserTerm,
         wait::{Wait, WaitError, WaitOptions},
-        Group, User,
     },
 };
 
@@ -65,12 +64,13 @@ pub struct RunOptions<'a> {
     pub command: &'a Path,
     pub arguments: &'a [String],
     pub arg0: Option<&'a Path>,
-    pub chdir: Option<&'a Path>,
+    pub chdir: Option<PathBuf>,
     pub is_login: bool,
     pub user: &'a User,
     pub group: &'a Group,
     pub umask: Umask,
 
+    pub background: bool,
     pub use_pty: bool,
     pub noexec: bool,
 }
@@ -83,6 +83,19 @@ pub fn run_command(
     options: RunOptions<'_>,
     env: impl IntoIterator<Item = (impl AsRef<OsStr>, impl AsRef<OsStr>)>,
 ) -> io::Result<ExitReason> {
+    if options.background {
+        // SAFETY: There should be no other threads at this point.
+        match unsafe { fork() }? {
+            ForkResult::Parent(_) => process::exit(0),
+            ForkResult::Child => {
+                // Child continues in an orphaned process group.
+                // Reads from the terminal fail with EIO.
+                // Writes succeed unless tostop is set on the terminal.
+                setpgid(ProcessId::new(0), ProcessId::new(0))?;
+            }
+        }
+    }
+
     // FIXME: should we pipe the stdio streams?
     let qualified_path = options.command;
     let mut command = Command::new(qualified_path);
@@ -112,7 +125,7 @@ pub fn run_command(
         ));
 
         #[cfg(target_os = "linux")]
-        Some(noexec::add_noexec_filter(&mut command))
+        Some(noexec::add_noexec_filter(&mut command)?)
     } else {
         None
     };
@@ -120,6 +133,7 @@ pub fn run_command(
     // Decide if the pwd should be changed. `--chdir` takes precedence over `-i`.
     let path = options
         .chdir
+        .as_ref()
         .map(|chdir| chdir.to_owned())
         .or_else(|| options.is_login.then(|| options.user.home.clone().into()))
         .clone();
@@ -136,7 +150,11 @@ pub fn run_command(
         unsafe {
             command.pre_exec(move || {
                 if let Err(err) = env::set_current_dir(&path) {
-                    user_error!("unable to change directory to {}: {}", path.display(), err);
+                    user_error!(
+                        "unable to change directory to {path}: {error}",
+                        path = path.display(),
+                        error = err
+                    );
                     if is_chdir {
                         return Err(err);
                     }
@@ -174,7 +192,14 @@ pub fn run_command(
 
     if options.use_pty {
         match UserTerm::open() {
-            Ok(user_tty) => exec_pty(sudo_pid, spawn_noexec_handler, command, user_tty),
+            Ok(user_tty) => exec_pty(
+                sudo_pid,
+                spawn_noexec_handler,
+                command,
+                user_tty,
+                options.user,
+                options.background,
+            ),
             Err(err) => {
                 dev_info!("Could not open user's terminal, not allocating a pty: {err}");
                 exec_no_pty(sudo_pid, spawn_noexec_handler, command)
@@ -195,8 +220,14 @@ pub enum ExitReason {
 fn exec_command(
     mut command: Command,
     original_set: Option<SignalSet>,
+    mut original_signal: SignalsState,
     mut errpipe_tx: BinPipe<i32>,
 ) -> ! {
+    // Restore the signal handlers of modified signals
+    if let Err(err) = original_signal.restore() {
+        dev_warn!("cannot restore signal states: {err}");
+    }
+
     // Restore the signal mask now that the handlers have been setup.
     if let Some(set) = original_set {
         if let Err(err) = set.set_mask() {
