@@ -1,8 +1,12 @@
 use std::collections::VecDeque;
 use std::ffi::c_int;
 use std::io;
+use std::os::fd::{FromRawFd, OwnedFd};
 use std::process::{Command, Stdio};
 
+use libc::{close, O_CLOEXEC};
+
+use crate::cutils::cerr;
 use crate::exec::event::{EventHandle, EventRegistry, PollEvent, Process, StopReason};
 use crate::exec::use_pty::monitor::exec_monitor;
 use crate::exec::use_pty::SIGCONT_FG;
@@ -15,7 +19,7 @@ use crate::exec::{
 use crate::log::{dev_error, dev_info, dev_warn};
 use crate::system::signal::{
     consts::*, register_handlers, SignalHandler, SignalHandlerBehavior, SignalNumber, SignalSet,
-    SignalStream,
+    SignalStream, SignalsState,
 };
 use crate::system::term::{Pty, PtyFollower, PtyLeader, TermSize, Terminal, UserTerm};
 use crate::system::wait::WaitOptions;
@@ -30,9 +34,13 @@ pub(in crate::exec) fn exec_pty(
     spawn_noexec_handler: Option<SpawnNoexecHandler>,
     mut command: Command,
     user_tty: UserTerm,
+    pty_owner: &User,
+    background: bool,
 ) -> io::Result<ExitReason> {
     // Allocate a pseudoterminal.
-    let pty = get_pty()?;
+    let pty = get_pty(pty_owner)?;
+
+    let mut original_signals = SignalsState::save()?;
 
     // Create backchannels to communicate with the monitor.
     let mut backchannels = BackchannelPair::new().map_err(|err| {
@@ -41,11 +49,19 @@ pub(in crate::exec) fn exec_pty(
     })?;
 
     // We don't want to receive SIGTTIN/SIGTTOU
-    match SignalHandler::register(SIGTTIN, SignalHandlerBehavior::Ignore) {
+    match SignalHandler::register(
+        SIGTTIN,
+        SignalHandlerBehavior::Ignore,
+        &mut original_signals,
+    ) {
         Ok(handler) => handler.forget(),
         Err(err) => dev_warn!("cannot set handler for SIGTTIN: {err}"),
     }
-    match SignalHandler::register(SIGTTOU, SignalHandlerBehavior::Ignore) {
+    match SignalHandler::register(
+        SIGTTOU,
+        SignalHandlerBehavior::Ignore,
+        &mut original_signals,
+    ) {
         Ok(handler) => handler.forget(),
         Err(err) => dev_warn!("cannot set handler for SIGTTOU: {err}"),
     }
@@ -54,7 +70,7 @@ pub(in crate::exec) fn exec_pty(
     // `policy_init_session`.
     // FIXME (ogsudo): initializes ttyblock sigset here by calling `init_ttyblock`
 
-    // Fetch the parent process group so we can signals to it.
+    // Fetch the parent process group so we can send signals to it.
     let parent_pgrp = getpgrp();
 
     // Set all the IO streams for the command to the follower side of the pty.
@@ -80,6 +96,10 @@ pub(in crate::exec) fn exec_pty(
         ParentEvent::Pty,
     );
 
+    if background {
+        tty_pipe.disable_input(&mut registry);
+    }
+
     let user_tty = tty_pipe.left_mut();
 
     // Check if we are the foreground process
@@ -93,21 +113,44 @@ pub(in crate::exec) fn exec_pty(
 
     // FIXME: maybe all these boolean flags should be on a dedicated type.
 
-    // Whether we're running on a pipeline
-    let mut pipeline = false;
     // Whether the command should be executed in the background (this is not the `-b` flag)
     let mut exec_bg = false;
     // Whether the user's terminal is in raw mode or not.
     let mut term_raw = false;
+    // Whether to preserve oflag for the terminal
+    let mut preserve_oflag = false;
 
     // Check if we are part of a pipeline.
     // FIXME: Here's where we should intercept the IO streams if we want to implement IO logging.
     // FIXME: ogsudo creates pipes for the IO streams and uses events to read from the strams to
     // the pipes. Investigate why.
-    if !io::stdin().is_terminal() {
-        dev_info!("stdin is not a terminal, command will inherit it");
-        pipeline = true;
-        command.stdin(Stdio::inherit());
+    if !io::stdin().is_terminal_for_pgrp(parent_pgrp) {
+        if background {
+            // Running in background (sudo -b), no access to terminal input.
+            // In non-pty mode, the command runs in an orphaned process
+            // group and reads from the controlling terminal fail with EIO.
+            // We cannot do the same while running in a pty but if we set
+            // stdin to a half-closed pipe, reads from it will get EOF.
+            dev_info!("stdin is not a terminal, creating a pipe");
+            exec_bg = true;
+
+            let mut pipes = [-1, -1];
+            // SAFETY: A valid pointer to a mutable array of 2 fds is passed in.
+            unsafe {
+                cerr(libc::pipe2(pipes.as_mut_ptr(), O_CLOEXEC))?;
+            }
+            // SAFETY: pipe2 created two owned pipe fds.
+            unsafe {
+                command.stdin(OwnedFd::from_raw_fd(pipes[0]));
+                close(pipes[1]);
+            }
+        } else {
+            dev_info!("stdin is not a terminal, command will inherit it");
+            if io::stdin().is_pipe_or_socket() {
+                exec_bg = true;
+            }
+            command.stdin(Stdio::inherit());
+        }
 
         if foreground && parent_pgrp != sudo_pid {
             // If sudo is not the process group leader and stdin is not a terminal we might be
@@ -117,20 +160,23 @@ pub(in crate::exec) fn exec_pty(
         }
     }
 
-    if !io::stdout().is_terminal() {
+    if !io::stdout().is_terminal_for_pgrp(parent_pgrp) {
         dev_info!("stdout is not a terminal, command will inherit it");
-        pipeline = true;
+        if io::stdout().is_pipe_or_socket() {
+            exec_bg = true;
+            preserve_oflag = true;
+        }
         command.stdout(Stdio::inherit());
     }
 
-    if !io::stderr().is_terminal() {
+    if !io::stderr().is_terminal_for_pgrp(parent_pgrp) {
         dev_info!("stderr is not a terminal, command will inherit it");
         command.stderr(Stdio::inherit());
     }
 
     // If there is another process later in the pipeline, don't interfere
     // with its access to the Tty
-    if io::stdout().is_pipe() {
+    if io::stdout().is_pipe_or_socket() {
         foreground = false;
     }
 
@@ -141,10 +187,10 @@ pub(in crate::exec) fn exec_pty(
     }
 
     // Start in raw mode unless we're part of a pipeline or backgrounded.
-    if foreground && !pipeline && !exec_bg {
+    if foreground && !exec_bg {
         // Clearer this way that set_raw_mode only conditionally runs
         #[allow(clippy::collapsible_if)]
-        if user_tty.set_raw_mode(false).is_ok() {
+        if user_tty.set_raw_mode(false, preserve_oflag).is_ok() {
             term_raw = true;
         }
     }
@@ -182,9 +228,10 @@ pub(in crate::exec) fn exec_pty(
         match exec_monitor(
             pty.follower,
             command,
-            foreground && !pipeline && !exec_bg,
+            foreground && !exec_bg,
             &mut backchannels.monitor,
             original_set,
+            original_signals,
         ) {
             Ok(exec_output) => match exec_output {},
             Err(err) => {
@@ -231,7 +278,9 @@ pub(in crate::exec) fn exec_pty(
         tty_size,
         foreground,
         term_raw,
+        preserve_oflag,
         &mut registry,
+        original_signals,
     )?;
 
     // Restore the signal mask now that the handlers have been setup.
@@ -267,7 +316,7 @@ pub(in crate::exec) fn exec_pty(
     exit_reason
 }
 
-fn get_pty() -> io::Result<Pty> {
+fn get_pty(pty_owner: &User) -> io::Result<Pty> {
     let tty_gid = Group::from_name(cstr!("tty"))
         .unwrap_or(None)
         .map(|group| group.gid);
@@ -277,9 +326,8 @@ fn get_pty() -> io::Result<Pty> {
         io::Error::new(io::ErrorKind::NotFound, "unable to open pty")
     })?;
 
-    let euid = User::effective_uid();
     let gid = tty_gid.unwrap_or(User::effective_gid());
-    chown(&pty.path, euid, gid).map_err(|err| {
+    chown(&pty.path, pty_owner.uid, gid).map_err(|err| {
         dev_error!("cannot change owner for pty: {err}");
         err
     })?;
@@ -299,9 +347,11 @@ struct ParentClosure {
     tty_size: TermSize,
     foreground: bool,
     term_raw: bool,
+    preserve_oflag: bool,
     backchannel: ParentBackchannel,
     message_queue: VecDeque<MonitorMessage>,
     backchannel_write_handle: EventHandle,
+    original_signals: SignalsState,
     signal_stream: &'static SignalStream,
     signal_handlers: [SignalHandler; ParentClosure::SIGNALS.len()],
 }
@@ -322,7 +372,9 @@ impl ParentClosure {
         tty_size: TermSize,
         foreground: bool,
         term_raw: bool,
+        preserve_oflag: bool,
         registry: &mut EventRegistry<Self>,
+        mut original_signals: SignalsState,
     ) -> io::Result<Self> {
         // Enable nonblocking assertions as we will poll this inside the event loop.
         backchannel.set_nonblocking_asserts(true);
@@ -338,7 +390,7 @@ impl ParentClosure {
 
         registry.register_event(signal_stream, PollEvent::Readable, |_| ParentEvent::Signal);
 
-        let signal_handlers = register_handlers(Self::SIGNALS)?;
+        let signal_handlers = register_handlers(Self::SIGNALS, &mut original_signals)?;
 
         Ok(Self {
             monitor_pid: Some(monitor_pid),
@@ -349,9 +401,11 @@ impl ParentClosure {
             tty_size,
             foreground,
             term_raw,
+            preserve_oflag,
             backchannel,
             message_queue: VecDeque::new(),
             backchannel_write_handle,
+            original_signals,
             signal_stream,
             signal_handlers,
         })
@@ -512,9 +566,13 @@ impl ParentClosure {
         registry: &mut EventRegistry<Self>,
     ) -> Option<SignalNumber> {
         // Ignore `SIGCONT` while suspending to avoid resuming the terminal twice.
-        let sigcont_handler = SignalHandler::register(SIGCONT, SignalHandlerBehavior::Ignore)
-            .map_err(|err| dev_warn!("cannot set handler for SIGCONT: {err}"))
-            .ok();
+        let sigcont_handler = SignalHandler::register(
+            SIGCONT,
+            SignalHandlerBehavior::Ignore,
+            &mut self.original_signals,
+        )
+        .map_err(|err| dev_warn!("cannot set handler for SIGCONT: {err}"))
+        .ok();
 
         if let SIGTTOU | SIGTTIN = signal {
             // If sudo is already the foreground process we can resume the command in the
@@ -530,10 +588,16 @@ impl ParentClosure {
                     signal_fmt(signal)
                 );
                 if !self.term_raw {
-                    if self.tty_pipe.left_mut().set_raw_mode(false).is_ok() {
+                    if self
+                        .tty_pipe
+                        .left_mut()
+                        .set_raw_mode(false, self.preserve_oflag)
+                        .is_ok()
+                    {
                         self.term_raw = true;
                     }
                     // Resume command in the foreground
+                    self.tty_pipe.enable_input(registry);
                     return Some(SIGCONT_FG);
                 }
             }
@@ -550,9 +614,13 @@ impl ParentClosure {
         }
 
         let signal_handler = if signal != SIGSTOP {
-            SignalHandler::register(signal, SignalHandlerBehavior::Default)
-                .map_err(|err| dev_warn!("cannot set handler for {}: {err}", signal_fmt(signal)))
-                .ok()
+            SignalHandler::register(
+                signal,
+                SignalHandlerBehavior::Default,
+                &mut self.original_signals,
+            )
+            .map_err(|err| dev_warn!("cannot set handler for {}: {err}", signal_fmt(signal)))
+            .ok()
         } else {
             None
         };
@@ -613,7 +681,12 @@ impl ParentClosure {
 
         if self.foreground {
             // We're in the foreground, set tty to raw mode.
-            if self.tty_pipe.left_mut().set_raw_mode(false).is_ok() {
+            if self
+                .tty_pipe
+                .left_mut()
+                .set_raw_mode(false, self.preserve_oflag)
+                .is_ok()
+            {
                 self.term_raw = true;
             }
         } else {
