@@ -1,21 +1,20 @@
 use std::{
-    ffi::{CStr, CString, OsStr, OsString},
+    ffi::{c_int, c_void, CStr, CString, OsStr, OsString},
     io,
     os::raw::c_char,
     os::unix::prelude::OsStrExt,
     ptr::NonNull,
-};
-
-use crate::system::{
-    signal::{self, SignalSet},
     time::Duration,
 };
 
+use crate::system::signal::{self, SignalSet};
+
 use converse::ConverserData;
 use error::pam_err;
 pub use error::{PamError, PamErrorType, PamResult};
 use sys::*;
 
+mod askpass;
 mod converse;
 mod error;
 mod rpassword;
@@ -26,6 +25,9 @@ mod securemem;
 #[allow(nonstandard_style)]
 pub mod sys;
 
+#[link(name = "pam")]
+extern "C" {}
+
 #[cfg(target_os = "freebsd")]
 const PAM_DATA_SILENT: std::ffi::c_int = 0;
 
@@ -36,7 +38,7 @@ pub struct PamContext {
     pamh: *mut pam_handle_t,
     silent: bool,
     allow_null_auth_token: bool,
-    last_pam_status: Option<libc::c_int>,
+    last_pam_status: Option<c_int>,
     session_started: bool,
 }
 
@@ -52,6 +54,7 @@ impl PamContext {
     pub fn new_cli(
         converser_name: &str,
         service_name: &str,
+        use_askpass: bool,
         use_stdin: bool,
         bell: bool,
         no_interact: bool,
@@ -62,6 +65,7 @@ impl PamContext {
         let converser = CLIConverser {
             bell,
             name: converser_name.to_owned(),
+            use_askpass,
             use_stdin,
             password_feedback,
             password_timeout,
@@ -79,8 +83,8 @@ impl PamContext {
             converser,
             converser_name: converser_name.to_owned(),
             no_interact,
-            auth_prompt: Some("authenticate".to_owned()),
-            timed_out: false,
+            auth_prompt: Some(xlat!("authenticate").to_owned()),
+            error: None,
             panicked: false,
         }));
 
@@ -93,7 +97,7 @@ impl PamContext {
                 c_user_ptr,
                 &pam_conv {
                     conv: Some(converse::converse::<CLIConverser>),
-                    appdata_ptr: data_ptr as *mut libc::c_void,
+                    appdata_ptr: data_ptr as *mut c_void,
                 },
                 &mut pamh,
             )
@@ -176,8 +180,8 @@ impl PamContext {
         }
 
         // SAFETY: self.data_ptr was created by Box::into_raw
-        if unsafe { (*self.data_ptr).timed_out } {
-            return Err(PamError::TimedOut);
+        if let Some(error) = unsafe { (*self.data_ptr).error.take() } {
+            return Err(error);
         }
 
         #[allow(clippy::question_mark)]
@@ -229,13 +233,7 @@ impl PamContext {
         let c_user = CString::new(user)?;
         // SAFETY: `self.pamh` contains a correct handle (obtained from `pam_start`); furthermore,
         // `c_user.as_ptr()` will point to a correct null-terminated string.
-        pam_err(unsafe {
-            pam_set_item(
-                self.pamh,
-                PAM_USER as _,
-                c_user.as_ptr() as *const libc::c_void,
-            )
-        })
+        pam_err(unsafe { pam_set_item(self.pamh, PAM_USER as _, c_user.as_ptr() as *const c_void) })
     }
 
     /// Get the user that is currently active in the PAM handle
@@ -265,13 +263,7 @@ impl PamContext {
         let data = CString::new(tty_path.as_ref().as_bytes())?;
         // SAFETY: `self.pamh` contains a correct handle (obtained from `pam_start`); furthermore,
         // `data.as_ptr()` will point to a correct null-terminated string.
-        pam_err(unsafe {
-            pam_set_item(
-                self.pamh,
-                PAM_TTY as _,
-                data.as_ptr() as *const libc::c_void,
-            )
-        })
+        pam_err(unsafe { pam_set_item(self.pamh, PAM_TTY as _, data.as_ptr() as *const c_void) })
     }
 
     // Set the user that requested the actions in this PAM instance.
@@ -279,22 +271,16 @@ impl PamContext {
         let data = CString::new(user.as_bytes())?;
         // SAFETY: `self.pamh` contains a correct handle (obtained from `pam_start`); furthermore,
         // `data.as_ptr()` will point to a correct null-terminated string.
-        pam_err(unsafe {
-            pam_set_item(
-                self.pamh,
-                PAM_RUSER as _,
-                data.as_ptr() as *const libc::c_void,
-            )
-        })
+        pam_err(unsafe { pam_set_item(self.pamh, PAM_RUSER as _, data.as_ptr() as *const c_void) })
     }
 
     /// Re-initialize the credentials stored in PAM
     pub fn credentials_reinitialize(&mut self) -> PamResult<()> {
-        self.credentials(PAM_REINITIALIZE_CRED as libc::c_int)
+        self.credentials(PAM_REINITIALIZE_CRED as c_int)
     }
 
     /// Updates to the credentials stored in PAM
-    fn credentials(&mut self, action: libc::c_int) -> PamResult<()> {
+    fn credentials(&mut self, action: c_int) -> PamResult<()> {
         let mut flags = action;
         flags |= self.silent_flag();
 
@@ -311,7 +297,7 @@ impl PamContext {
         let mut flags = 0;
         flags |= self.silent_flag();
         if expired_only {
-            flags |= PAM_CHANGE_EXPIRED_AUTHTOK as libc::c_int;
+            flags |= PAM_CHANGE_EXPIRED_AUTHTOK as c_int;
         }
         // SAFETY: `self.pamh` contains a correct handle (obtained from `pam_start`).
         pam_err(unsafe { pam_chauthtok(self.pamh, flags) })
@@ -411,8 +397,7 @@ impl Drop for PamContext {
         unsafe {
             pam_end(
                 self.pamh,
-                self.last_pam_status.unwrap_or(PAM_SUCCESS as libc::c_int)
-                    | PAM_DATA_SILENT as libc::c_int,
+                self.last_pam_status.unwrap_or(PAM_SUCCESS as c_int) | PAM_DATA_SILENT as c_int,
             )
         };
     }
