@@ -1,12 +1,11 @@
 use crate::common::resolve::CurrentUser;
 use crate::common::{CommandAndArguments, Context};
-use crate::exec::Umask;
 use crate::sudo::{
     cli::{SudoAction, SudoRunOptions},
     env::environment::{get_target_environment, Environment},
 };
 use crate::system::interface::{GroupId, UserId};
-use crate::system::{Group, Hostname, Process, User};
+use crate::system::{Group, Hostname, User};
 use std::collections::{HashMap, HashSet};
 
 const TESTS: &str = "
@@ -29,7 +28,6 @@ const TESTS: &str = "
     LANGUAGE=en_US.UTF-8
     LC_ALL=en_US.UTF-8
     LS_COLORS=cd=40;33;01:*.jpg=01;35:*.mp3=00;36:
-    MAIL=/var/mail/root
     PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin
     SHELL=/bin/bash
     SUDO_COMMAND=/usr/bin/env
@@ -47,7 +45,6 @@ const TESTS: &str = "
     LANGUAGE=en_US.UTF-8
     LC_ALL=en_US.UTF-8
     LS_COLORS=cd=40;33;01:*.jpg=01;35:*.mp3=00;36:
-    MAIL=/var/mail/test
     PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin
     SHELL=/bin/sh
     SUDO_COMMAND=/usr/bin/env
@@ -129,16 +126,13 @@ fn create_test_context(sudo_options: SudoRunOptions) -> Context {
         },
         launch: crate::common::context::LaunchType::Direct,
         chdir: sudo_options.chdir,
+        askpass: sudo_options.askpass,
         stdin: sudo_options.stdin,
         prompt: sudo_options.prompt,
         non_interactive: sudo_options.non_interactive,
-        process: Process::new(),
         use_session_records: false,
-        use_pty: true,
-        noexec: false,
         bell: false,
-        umask: Umask::Preserve,
-        noninteractive_auth: false,
+        background: false,
         files_to_edit: vec![],
     }
 }
@@ -177,7 +171,6 @@ fn test_environment_variable_filtering() {
                 chdir: crate::sudoers::DirChange::Strict(None),
                 trust_environment: false,
                 umask: crate::exec::Umask::Preserve,
-                noninteractive_auth: false,
                 #[cfg(feature = "apparmor")]
                 apparmor_profile: None,
                 noexec: false,
