@@ -33,7 +33,7 @@ impl Token for Username {
 
     const ALLOW_ESCAPE: bool = true;
     fn escaped(c: char) -> bool {
-        matches!(c, '\\' | '"' | ',' | ':' | '=' | '!' | '(' | ')')
+        matches!(c, '\\' | '"' | ',' | ':' | '=' | '!' | '(' | ')' | ' ')
     }
 }
 
@@ -100,13 +100,6 @@ pub enum Meta<T> {
     Alias(String) = HARDENED_ENUM_VALUE_2,
 }
 
-impl<T> Meta<T> {
-    #[cfg(test)]
-    pub fn is_alias(&self) -> bool {
-        matches!(self, Self::Alias(..))
-    }
-}
-
 impl<T: Token> Token for Meta<T> {
     fn construct(raw: String) -> Result<Self, String> {
         // `T` may accept whitespace resulting in `raw` having trailing whitespace which would make
@@ -185,7 +178,13 @@ impl Token for Command {
             // if no arguments are mentioned, anything is allowed
             None
         } else {
-            if args.last().map(|x| -> &str { x }) == Some("\"\"") {
+            if args.first().is_some_and(|x| x.starts_with('^')) {
+                // regular expressions are not supported, give an error message. If there is only a
+                // terminating '$', this is not treated as a malformed regex by millersudo, so we don't
+                // need to seperately check for that
+                return Err("regular expressions are not supported".to_string());
+            }
+            if args.last().is_some_and(|x| x == "\"\"") {
                 // if the magic "" appears, no (further) arguments are allowed
                 args.pop();
             }
@@ -227,6 +226,8 @@ impl Token for SimpleCommand {
             return cvt_err(glob::Pattern::new(&cmd));
         } else if cmd.starts_with("sha") {
             return Err("digest specifications are not supported".to_string());
+        } else if cmd.starts_with('^') {
+            return Err("regular expressions are not supported".to_string());
         } else if !cmd.starts_with('/') {
             return Err("fully qualified path needed".to_string());
         }
@@ -259,7 +260,8 @@ impl Token for SimpleCommand {
     }
 
     fn accept(c: char) -> bool {
-        !Self::escaped(c) && !c.is_control()
+        // '=' is allowed both escaped and un-escaped
+        (!Self::escaped(c) && !c.is_control()) || c == '='
     }
 
     const ALLOW_ESCAPE: bool = true;
@@ -296,7 +298,7 @@ impl Token for EnvVar {
 
     const ALLOW_ESCAPE: bool = true;
     fn escaped(c: char) -> bool {
-        matches!(c, '\\' | '=' | '#' | '"')
+        matches!(c, '\\' | '=' | '#' | '"' | ',')
     }
 }
 
