@@ -1,38 +1,30 @@
-use core::fmt;
 // TODO: remove unused attribute when system is cleaned up
 #[cfg(target_os = "linux")]
 use std::str::FromStr;
 use std::{
-    collections::BTreeSet,
-    ffi::{c_uint, CStr, CString},
-    io,
+    ffi::{c_char, c_int, c_long, c_uint, CStr},
+    fmt, fs, io,
     mem::MaybeUninit,
     ops,
-    os::{
-        fd::{AsFd, AsRawFd},
-        unix::{self, prelude::OsStrExt},
-    },
-    path::{Path, PathBuf},
+    os::unix,
+    path::PathBuf,
 };
 
 use crate::{
     common::{Error, SudoPath, SudoString},
     cutils::*,
 };
-pub use audit::secure_open;
 use interface::{DeviceId, GroupId, ProcessId, UserId};
 pub use libc::PATH_MAX;
-use libc::STDERR_FILENO;
+use libc::{CLOSE_RANGE_CLOEXEC, EINVAL, ENOSYS, STDERR_FILENO};
 use time::ProcessCreateTime;
 
 use self::signal::SignalNumber;
 
-mod audit;
+pub(crate) mod audit;
 // generalized traits for when we want to hide implementations
 pub mod interface;
 
-pub mod kernel;
-
 pub mod file;
 
 pub mod time;
@@ -45,87 +37,80 @@ pub mod term;
 
 pub mod wait;
 
-pub(crate) fn can_execute<P: AsRef<Path>>(path: P) -> bool {
-    let Ok(path) = CString::new(path.as_ref().as_os_str().as_bytes()) else {
-        return false;
-    };
+#[cfg(not(any(target_os = "freebsd", target_os = "linux")))]
+compile_error!("sudo-rs only works on Linux and FreeBSD");
 
-    // SAFETY: we are passing a proper pointer to access
-    unsafe { libc::access(path.as_ptr(), libc::X_OK) == 0 }
-}
-
-pub(crate) fn _exit(status: libc::c_int) -> ! {
+pub(crate) fn _exit(status: c_int) -> ! {
     // SAFETY: this function is safe to call
     unsafe { libc::_exit(status) }
 }
 
-/// A type able to close every file descriptor except for the ones pased via [`FileCloser::except`]
-/// and the IO streams.
-pub(crate) struct FileCloser {
-    fds: BTreeSet<c_uint>,
-}
+/// Mark every file descriptor that is not one of the IO streams as CLOEXEC.
+pub(crate) fn mark_fds_as_cloexec() -> io::Result<()> {
+    let lowfd = STDERR_FILENO + 1;
 
-impl FileCloser {
-    pub(crate) const fn new() -> Self {
-        Self {
-            fds: BTreeSet::new(),
+    // SAFETY: this function is safe to call:
+    // - any errors while closing a specific fd will be effectively ignored
+    #[allow(clippy::diverging_sub_expression)]
+    let res = unsafe {
+        'a: {
+            #[cfg(not(target_os = "linux"))]
+            break 'a cerr(libc::close_range(
+                lowfd as c_uint,
+                c_uint::MAX,
+                CLOSE_RANGE_CLOEXEC as c_int,
+            ));
+            // on Linux, close_range was only added in glibc 2.34, and is not
+            // part of musl, so we go perform a straight syscall instead
+            #[cfg(target_os = "linux")]
+            break 'a cerr(libc::syscall(
+                libc::SYS_close_range,
+                lowfd as c_uint,
+                c_uint::MAX,
+                CLOSE_RANGE_CLOEXEC as c_uint,
+            ));
         }
-    }
+    };
 
-    pub(crate) fn except<F: AsFd>(&mut self, fd: &F) {
-        self.fds.insert(fd.as_fd().as_raw_fd() as c_uint);
-    }
+    match res {
+        Err(err) if err.raw_os_error() == Some(ENOSYS) || err.raw_os_error() == Some(EINVAL) => {
+            // The kernel doesn't support close_range or CLOSE_RANGE_CLOEXEC,
+            // fallback to finding all open fds using /proc/self/fd.
 
-    /// Close every file descriptor that is not one of the IO streams or one of the file
-    /// descriptors passed via [`FileCloser::except`].
-    ///
-    /// # Safety
-    ///
-    /// Incorrect use of this method can violate [I/O Safety](https://doc.rust-lang.org/std/io/index.html#io-safety)
-    /// by closing fds that are not owned by `FileCloser`. The caller needs to ensure that none of
-    /// the closed fds are ever accessed again.
-    // FIXME do not return a Result. This makes it way to easy to accidentally violate the safety conditions.
-    pub(crate) unsafe fn close_the_universe(self) -> io::Result<()> {
-        let mut fds = self.fds.into_iter();
-
-        let Some(mut curr_fd) = fds.next() else {
-            return close_range(STDERR_FILENO as c_uint + 1, c_uint::MAX);
-        };
-
-        if let Some(max_fd) = curr_fd.checked_sub(1) {
-            close_range(STDERR_FILENO as c_uint + 1, max_fd)?;
-        }
-
-        for next_fd in fds {
-            if let Some(min_fd) = curr_fd.checked_add(1) {
-                if let Some(max_fd) = next_fd.checked_sub(1) {
-                    close_range(min_fd, max_fd)?;
+            // FIXME use /dev/fd on macOS
+            for entry in fs::read_dir("/proc/self/fd")? {
+                let entry = entry?;
+                let file_name = entry.file_name();
+                let file_name = file_name.to_str().ok_or(io::Error::new(
+                    io::ErrorKind::InvalidData,
+                    "procfs returned non-integer fd name",
+                ))?;
+                if file_name == "." || file_name == ".." {
+                    continue;
+                }
+                let fd = file_name.parse::<c_int>().map_err(|_| {
+                    io::Error::new(
+                        io::ErrorKind::InvalidData,
+                        "procfs returned non-integer fd name",
+                    )
+                })?;
+                if fd < lowfd {
+                    continue;
+                }
+                // SAFETY: This only sets the CLOEXEC flag for the given fd. Nothing is
+                // going to need it after exec.
+                unsafe {
+                    cerr(libc::fcntl(fd, libc::F_SETFD, libc::FD_CLOEXEC))?;
                 }
             }
 
-            curr_fd = next_fd;
+            Ok(())
         }
-
-        if let Some(min_fd) = curr_fd.checked_add(1) {
-            close_range(min_fd, c_uint::MAX)?;
-        }
-
-        Ok(())
+        Err(err) => Err(err),
+        Ok(_) => Ok(()),
     }
 }
 
-fn close_range(min_fd: c_uint, max_fd: c_uint) -> io::Result<()> {
-    if min_fd <= max_fd {
-        // SAFETY: this function is safe to call:
-        // - any errors while closing a specific fd will be effectively ignored
-        // - if the provided range or flags are invalid, that will be reported
-        //   as an error but will not cause undefined behaviour
-        cerr(unsafe { libc::close_range(min_fd, max_fd, 0) })?;
-    }
-
-    Ok(())
-}
-
 pub(crate) enum ForkResult {
     // Parent process branch with the child process' PID.
     Parent(ProcessId),
@@ -227,7 +212,7 @@ impl Hostname {
 
     pub fn resolve() -> Self {
         // see `man 2 gethostname`
-        const MAX_HOST_NAME_SIZE_ACCORDING_TO_SUSV2: libc::c_long = 255;
+        const MAX_HOST_NAME_SIZE_ACCORDING_TO_SUSV2: c_long = 255;
 
         // POSIX.1 systems limit hostnames to `HOST_NAME_MAX` bytes
         // not including null-byte in the count
@@ -254,8 +239,8 @@ impl Hostname {
     }
 }
 
-pub fn syslog(priority: libc::c_int, facility: libc::c_int, message: &CStr) {
-    const MSG: *const libc::c_char = match CStr::from_bytes_until_nul(b"%s\0") {
+pub fn syslog(priority: c_int, facility: c_int, message: &CStr) {
+    const MSG: *const c_char = match CStr::from_bytes_until_nul(b"%s\0") {
         Ok(cstr) => cstr.as_ptr(),
         Err(_) => panic!("syslog formatting string is not null-terminated"),
     };
@@ -271,6 +256,33 @@ pub fn syslog(priority: libc::c_int, facility: libc::c_int, message: &CStr) {
     }
 }
 
+/// Makes sure that that the target is included in the groups, and is its first element
+fn inject_group(target: GroupId, groups: &mut Vec<GroupId>) {
+    if let Some(index) = groups.iter().position(|id| id == &target) {
+        // make sure the requested group id is the first in the list (necessary on FreeBSD)
+        groups.swap(0, index)
+    } else {
+        // add target group to list of additional groups if not present
+        groups.insert(0, target);
+    }
+}
+
+/// Set the supplementary groups -- returns a c_int to mimic a libc function
+fn set_supplementary_groups(groups: &[GroupId]) -> io::Result<()> {
+    // On FreeBSD, setgruops expects the size to be passed as a i32, so the below
+    // conversion protects a very extreme case of arithmetic conversion error
+    #[allow(irrefutable_let_patterns)]
+    #[allow(clippy::useless_conversion)]
+    let Ok(len) = groups.len().try_into() else {
+        return Err(io::Error::new(io::ErrorKind::Other, "too many groups"));
+    };
+    // SAFETY: setgroups is passed a valid pointer to a chunk of memory of the correct size
+    // We can cast to gid_t because `GroupId` is marked as transparent
+    cerr(unsafe { libc::setgroups(len, groups.as_ptr().cast::<libc::gid_t>()) })?;
+
+    Ok(())
+}
+
 /// set target user and groups (uid, gid, additional groups) for a command
 pub fn set_target_user(
     cmd: &mut std::process::Command,
@@ -279,17 +291,7 @@ pub fn set_target_user(
 ) {
     use std::os::unix::process::CommandExt;
 
-    if let Some(index) = target_user
-        .groups
-        .iter()
-        .position(|id| id == &target_group.gid)
-    {
-        // make sure the requested group id is the first in the list (necessary on FreeBSD)
-        target_user.groups.swap(0, index)
-    } else {
-        // add target group to list of additional groups if not present
-        target_user.groups.insert(0, target_group.gid);
-    }
+    inject_group(target_group.gid, &mut target_user.groups);
 
     // we need to do this in a `pre_exec` call since the `groups` method in `process::Command` is unstable
     // see https://github.com/rust-lang/rust/blob/a01b4cc9f375f1b95fa8195daeea938d3d9c4c34/library/std/src/sys/unix/process/process_unix.rs#L329-L352
@@ -297,11 +299,7 @@ pub fn set_target_user(
     // SAFETY: Setuid, setgid and setgroups are async-signal-safe.
     unsafe {
         cmd.pre_exec(move || {
-            cerr(libc::setgroups(
-                target_user.groups.len() as _,
-                // We can cast to gid_t because `GroupId` is marked as transparent
-                target_user.groups.as_ptr().cast::<libc::gid_t>(),
-            ))?;
+            set_supplementary_groups(&target_user.groups)?;
             // setgid and setuid set the real, effective and saved version of the gid and uid
             // respectively rather than just the real gid and uid. The original sudo uses setresgid
             // and setresuid instead with all three arguments equal, but as this does the same as
@@ -376,7 +374,7 @@ impl User {
     /// This function expects `pwd` to be a result from a successful call to `getpwXXX_r`.
     /// (It can cause UB if any of `pwd`'s pointed-to strings does not have a null-terminator.)
     unsafe fn from_libc(pwd: &libc::passwd) -> Result<User, Error> {
-        let mut buf_len: libc::c_int = 32;
+        let mut buf_len: c_int = 32;
         let mut groups_buffer: Vec<libc::gid_t>;
 
         while {
@@ -612,21 +610,13 @@ impl WithProcess {
 
 #[derive(Debug, Clone)]
 pub struct Process {
-    pub pid: ProcessId,
     pub parent_pid: Option<ProcessId>,
     pub session_id: ProcessId,
 }
 
-impl Default for Process {
-    fn default() -> Self {
-        Self::new()
-    }
-}
-
 impl Process {
     pub fn new() -> Process {
         Process {
-            pid: Self::process_id(),
             parent_pid: Self::parent_id(),
             session_id: Self::session_id(),
         }
@@ -676,10 +666,8 @@ impl Process {
         }
     }
 
-    /// Returns the device identifier of the TTY device that is currently
-    /// attached to the given process
     #[cfg(target_os = "freebsd")]
-    pub fn tty_device_id(pid: WithProcess) -> std::io::Result<Option<DeviceId>> {
+    fn get_proc_info(pid: WithProcess) -> std::io::Result<libc::kinfo_proc> {
         use std::ffi::c_void;
         use std::ptr;
 
@@ -692,6 +680,8 @@ impl Process {
 
         loop {
             let mut size = ki_proc.capacity() * size_of::<libc::kinfo_proc>();
+            // SAFETY: KERN_PROC_PID only reads data into the ki_proc list. It
+            // does not write more than `size` bytes to the pointer.
             match cerr(unsafe {
                 libc::sysctl(
                     [
@@ -727,10 +717,19 @@ impl Process {
             }
         }
 
-        if ki_proc[0].ki_tdev == !0 {
+        Ok(ki_proc[0])
+    }
+
+    /// Returns the device identifier of the TTY device that is currently
+    /// attached to the given process
+    #[cfg(target_os = "freebsd")]
+    pub fn tty_device_id(pid: WithProcess) -> std::io::Result<Option<DeviceId>> {
+        let ki_proc = Self::get_proc_info(pid)?;
+
+        if ki_proc.ki_tdev == !0 {
             Ok(None)
         } else {
-            Ok(Some(DeviceId::new(ki_proc[0].ki_tdev)))
+            Ok(Some(DeviceId::new(ki_proc.ki_tdev)))
         }
     }
 
@@ -758,57 +757,13 @@ impl Process {
     /// Get the process starting time of a specific process
     #[cfg(target_os = "freebsd")]
     pub fn starting_time(pid: WithProcess) -> io::Result<ProcessCreateTime> {
-        use std::ffi::c_void;
-        use std::ptr;
+        let ki_proc = Self::get_proc_info(pid)?;
 
-        let mut ki_proc: Vec<libc::kinfo_proc> = Vec::with_capacity(1);
-
-        let pid = match pid {
-            WithProcess::Current => std::process::id() as i32,
-            WithProcess::Other(pid) => pid.inner(),
-        };
-
-        loop {
-            let mut size = ki_proc.capacity() * size_of::<libc::kinfo_proc>();
-            match cerr(unsafe {
-                libc::sysctl(
-                    [
-                        libc::CTL_KERN,
-                        libc::KERN_PROC,
-                        libc::KERN_PROC_PID,
-                        pid,
-                        size_of::<libc::kinfo_proc>() as i32,
-                        1,
-                    ]
-                    .as_ptr(),
-                    4,
-                    ki_proc.as_mut_ptr().cast::<c_void>(),
-                    &mut size,
-                    ptr::null(),
-                    0,
-                )
-            }) {
-                Ok(_) => {
-                    assert!(size >= size_of::<libc::kinfo_proc>());
-                    // SAFETY: The above sysctl has initialized at least `size` bytes. We have
-                    // asserted that this is at least a single element.
-                    unsafe {
-                        ki_proc.set_len(1);
-                    }
-                    break;
-                }
-                Err(e) if e.raw_os_error() == Some(libc::ENOMEM) => {
-                    // Vector not big enough. Grow it by 10% and try again.
-                    ki_proc.reserve(ki_proc.capacity() + (ki_proc.capacity() + 9) / 10);
-                }
-                Err(e) => return Err(e),
-            }
-        }
-
-        let ki_start = ki_proc[0].ki_start;
+        let ki_start = ki_proc.ki_start;
+        #[allow(clippy::useless_conversion)]
         Ok(ProcessCreateTime::new(
-            ki_start.tv_sec,
-            ki_start.tv_usec * 1000,
+            i64::from(ki_start.tv_sec),
+            i64::from(ki_start.tv_usec) * 1000,
         ))
     }
 }
@@ -908,6 +863,7 @@ pub(crate) const ROOT_GROUP_NAME: &str = "wheel";
 #[cfg(test)]
 mod tests {
     use std::{
+        ffi::c_char,
         io::{self, Read, Write},
         os::{
             fd::{AsFd, AsRawFd},
@@ -933,7 +889,7 @@ mod tests {
             .as_nanos();
         let pid = std::process::id();
 
-        let filename = format!("sudo_rs_test_{}_{}", pid, timestamp);
+        let filename = format!("sudo_rs_test_{pid}_{timestamp}");
         let path = std::path::PathBuf::from("/tmp").join(filename);
         std::fs::File::options()
             .read(true)
@@ -992,7 +948,7 @@ mod tests {
                                 .iter()
                                 .map(|cs| cs.as_ptr() as *mut _)
                                 .chain(std::iter::once(std::ptr::null_mut()))
-                                .collect::<Vec<*mut libc::c_char>>()
+                                .collect::<Vec<*mut c_char>>()
                                 .as_mut_ptr(),
                         })
                     }
@@ -1014,14 +970,6 @@ mod tests {
         assert!(super::Process::tty_device_id(WithProcess::Current).is_ok());
     }
 
-    #[test]
-    fn get_process_start_time() {
-        let time = super::Process::starting_time(WithProcess::Current).unwrap();
-        let now = super::ProcessCreateTime::now().unwrap();
-        assert!(time.secs() > now.secs() - 24 * 60 * 60);
-        assert!(time < now);
-    }
-
     #[test]
     fn pgid_test() {
         use super::{getpgid, setpgid};
@@ -1096,60 +1044,33 @@ mod tests {
         );
     }
 
-    fn is_closed<F: AsFd>(fd: &F) -> bool {
-        crate::cutils::cerr(unsafe { libc::fcntl(fd.as_fd().as_raw_fd(), libc::F_GETFD) })
-            .is_err_and(|err| err.raw_os_error() == Some(libc::EBADF))
+    fn is_cloexec<F: AsFd>(fd: &F) -> bool {
+        crate::cutils::cerr(unsafe { libc::fcntl(fd.as_fd().as_raw_fd(), libc::F_GETFD) }).unwrap()
+            & libc::FD_CLOEXEC
+            == libc::FD_CLOEXEC
     }
 
     #[test]
-    fn close_the_universe() {
+    fn mark_fds_as_cloexec() {
         let child_pid = unsafe {
             fork_for_test(|| {
                 let should_close =
                     std::fs::File::create(std::env::temp_dir().join("should_close.txt")).unwrap();
-                assert!(!is_closed(&should_close));
+                crate::cutils::cerr(libc::fcntl(
+                    should_close.as_fd().as_raw_fd(),
+                    libc::F_SETFD,
+                    0,
+                ))
+                .unwrap();
+                assert!(!is_cloexec(&should_close));
 
-                let should_not_close =
-                    std::fs::File::create(std::env::temp_dir().join("should_not_close.txt"))
-                        .unwrap();
-                assert!(!is_closed(&should_not_close));
+                super::mark_fds_as_cloexec().unwrap();
 
-                let mut closer = super::FileCloser::new();
+                assert!(is_cloexec(&should_close));
 
-                closer.except(&should_not_close);
-
-                closer.close_the_universe().unwrap();
-
-                assert!(is_closed(&should_close));
-
-                assert!(!is_closed(&io::stdin()));
-                assert!(!is_closed(&io::stdout()));
-                assert!(!is_closed(&io::stderr()));
-                assert!(!is_closed(&should_not_close));
-
-                exit(0)
-            })
-        };
-
-        let (_, status) = child_pid.wait(WaitOptions::new()).unwrap();
-        assert_eq!(status.exit_status(), Some(0));
-    }
-
-    #[test]
-    fn except_stdio_is_fine() {
-        let child_pid = unsafe {
-            fork_for_test(|| {
-                let mut closer = super::FileCloser::new();
-
-                closer.except(&io::stdin());
-                closer.except(&io::stdout());
-                closer.except(&io::stderr());
-
-                closer.close_the_universe().unwrap();
-
-                assert!(!is_closed(&io::stdin()));
-                assert!(!is_closed(&io::stdout()));
-                assert!(!is_closed(&io::stderr()));
+                assert!(!is_cloexec(&io::stdin()));
+                assert!(!is_cloexec(&io::stdout()));
+                assert!(!is_cloexec(&io::stderr()));
 
                 exit(0)
             })
