@@ -2,7 +2,7 @@ use std::io;
 
 use crate::log::dev_warn;
 
-use super::{consts::*, set::SignalAction, signal_name, SignalNumber};
+use super::{consts::*, set::SignalAction, signal_name, SignalNumber, SignalsState};
 
 /// A handler for a signal.
 ///
@@ -21,9 +21,13 @@ impl SignalHandler {
     /// # Panics
     ///
     /// If it is not possible to override the action for the provided signal.
+    /// If `state` is `Some`, signal changes are tracked and will potentially restored in the
+    /// future.
+    /// If `None`, the caller explicitly opts out of restoration.
     pub(crate) fn register(
         signal: SignalNumber,
         behavior: SignalHandlerBehavior,
+        state: &mut SignalsState,
     ) -> io::Result<Self> {
         if Self::FORBIDDEN.contains(&signal) {
             panic!(
@@ -34,6 +38,7 @@ impl SignalHandler {
 
         let action = SignalAction::new(behavior)?;
         let original_action = action.register(signal)?;
+        state.updated(signal)?;
 
         Ok(Self {
             signal,
