@@ -2,15 +2,18 @@
 mod handler;
 mod info;
 mod set;
+mod state;
 mod stream;
 
 pub(crate) use handler::{SignalHandler, SignalHandlerBehavior};
 pub(crate) use set::SignalSet;
+pub(crate) use state::SignalsState;
 pub(crate) use stream::{register_handlers, SignalStream};
 
 use std::borrow::Cow;
+use std::ffi::c_int;
 
-pub(crate) type SignalNumber = libc::c_int;
+pub(crate) type SignalNumber = c_int;
 
 macro_rules! define_consts {
     ($($signal:ident,)*) => {
